class Event(dict):

    def serialize(self, kw='=', join=', '.join, quote=repr,
                  exclude_keys=frozenset()):
        return join(str(k)+kw+quote(v) for (k, v) in sorted(self.items())
                    if k not in exclude_keys)

    def __repr__(self):
        return 'Event('+self.serialize()+')'



from logging import WARNING, StreamHandler, getLogger
from unittest import TestCase

from testfixtures import LogCapture, OutputCapture, compare

from shoehorn import get_logger
from shoehorn.compat import PY2
from shoehorn.event import Event
from shoehorn.stdlib import StandardLibraryTarget, ShoehornFormatter



class TestStandardLibraryTarget(TestCase):

    def setUp(self):
        self.capture = LogCapture(
            attributes=('name', 'levelname', 'getMessage', 'shoehorn_event')
        )
        self.addCleanup(self.capture.uninstall)
        self.target = StandardLibraryTarget()

    def test_minimal(self):
        event = Event(event='test')
        self.target(event)
        self.capture.check(
            ('root', 'INFO', 'None', event)
        )

    def test_specifify_default_level(self):
        target = StandardLibraryTarget(default_level=WARNING)
        event = Event(event='test')
        target(event)
        self.capture.check(
            ('root', 'WARNING', 'None', event)
        )

    def test_named_logger(self):
        event = Event(event='test', logger='foo')
        self.target(event)
        self.capture.check(
            ('foo', 'INFO', 'None', event)
        )

    def test_numeric_level(self):
        event = Event(event='test', level=WARNING)
        self.target(event)
        self.capture.check(
            ('root', 'WARNING', 'None', event)
        )

    def test_string_level(self):
        event = Event(event='test', level='warning')
        self.target(event)
        self.capture.check(
            ('root', 'WARNING', 'None', event)
        )

    def test_unknown_string_level(self):
        event = Event(event='test', level='yuhwut?')
        self.target(event)
        self.capture.check(
            ('root', 'INFO', 'None', event)
        )

    def test_sub_args(self):
        event = Event(message='foo %s', args=('bar', ))
        self.target(event)
        self.capture.check(
            ('root', 'INFO', 'foo bar', event)
        )

    def test_exc_info(self):
        bad = Exception('bad')
        try:
            raise bad
        except:
            event = Event(level='error', message='foo', exc_info=True)
            self.target(event)
        self.capture.check(
            ('root', 'ERROR', 'foo', event)
        )
        compare(bad, actual=self.capture.records[-1].exc_info[1])

    def test_stack_info(self):
        if PY2:
            return
        event = Event(message='foo', stack_info=True)
        self.target(event)
        self.capture.check(
            ('root', 'INFO', 'foo', event)
        )
        compare('Stack (most recent call last):',
                actual=self.capture.records[-1].stack_info.split('\n')[0])


class TestShoehornFormatter(TestCase):

    def setUp(self):
        # so we don't leave a mess
        self.capture = LogCapture()
        self.addCleanup(self.capture.uninstall)
        with OutputCapture() as output:
            self.handler = StreamHandler()
        self.output = output
        self.handler.setFormatter(ShoehornFormatter())
        self.logger = getLogger()
        self.logger.addHandler(self.handler)

    def test_no_context(self):
        kw = dict(exc_info=True)
        if not PY2:
            kw['stack_info']=True
        try:
            1/0
        except:
            self.logger.info('foo %s', 'bar', **kw)
        compare(self.output.captured.splitlines()[0],
                expected='foo bar ')

    def test_extra_context(self):
        kw = dict(exc_info=True, context='oh hai', other=1)
        if not PY2:
            kw['stack_info']=True
        try:
            1/0
        except:
            get_logger().info('foo %s', 'bar', **kw)
        compare(self.output.captured.splitlines()[0],
                expected="foo bar context='oh hai' other=1")

    def test_bound_logger(self):
        self.handler.setFormatter(ShoehornFormatter(
            '%(name)s %(message)s%(shoehorn_context)s'
        ))
        get_logger('foo.bar').info('oh hai')
        compare(self.output.captured,
                expected="foo.bar oh hai\n")



from unittest import TestCase

from testfixtures import compare, ShouldRaise

from shoehorn import Logger
from shoehorn.event import Event


class TestLogger(TestCase):

    def setUp(self):
        self.records = []
        self.logger = Logger(self.records.append)

    def check(self, *expected):
        compare(expected, actual=self.records)

    def test_minimal(self):
        self.logger.debug(event='foo')
        self.logger.info(event='foo')
        self.logger.warning(event='foo')
        self.logger.warn(event='foo')
        self.logger.error(event='foo')
        self.logger.exception(event='foo')
        self.logger.critical(event='foo')
        self.logger.fatal(event='foo')
        self.logger.log(level=42, event='foo')
        self.check(
            Event(level='debug', event='foo'),
            Event(level='info', event='foo'),
            Event(level='warning', event='foo'),
            Event(level='warning', event='foo'),
            Event(level='error', event='foo'),
            Event(level='error', event='foo', exc_info=True),
            Event(level='critical', event='foo'),
            Event(level='critical', event='foo'),
            Event(level=42, event='foo'),
        )


    def test_stdlib(self):
        self.logger.debug('foo %s', 'bar',
                          exc_info=None, extra=None, stack_info=False)
        self.logger.info('foo %s', 'bar',
                         exc_info=None, extra=None, stack_info=False)
        self.logger.warning('foo %s', 'bar',
                            exc_info=None, extra=None, stack_info=False)
        self.logger.warn('foo %s', 'bar',
                         exc_info=None, extra=None, stack_info=False)
        self.logger.error('foo %s', 'bar',
                          exc_info=None, extra=None, stack_info=False)
        self.logger.exception('foo %s', 'bar',
                              exc_info=None, extra=None, stack_info=False)
        self.logger.critical('foo %s', 'bar',
                             exc_info=None, extra=None, stack_info=False)
        self.logger.fatal('foo %s', 'bar',
                          exc_info=None, extra=None, stack_info=False)
        self.logger.log(42, 'foo %s', 'bar',
                        exc_info=None, extra=None, stack_info=False)
        self.check(
            Event(level='debug', message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
            Event(level='info', message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
            Event(level='warning', message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
            Event(level='warning', message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
            Event(level='error', message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
            Event(level='error', message='foo %s', args=('bar', ),
                  exc_info=True, extra=None, stack_info=False),
            Event(level='critical', message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
            Event(level='critical', message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
            Event(level=42, message='foo %s', args=('bar', ),
                  exc_info=None, extra=None, stack_info=False),
        )

    def test_simple(self):
        self.logger.debug('foo')
        self.logger.info('foo')
        self.logger.warning('foo')
        self.logger.warn('foo')
        self.logger.error('foo')
        self.logger.exception('foo')
        self.logger.critical('foo')
        self.logger.fatal('foo')
        self.check(
            Event(level='debug', message='foo'),
            Event(level='info', message='foo'),
            Event(level='warning', message='foo'),
            Event(level='warning', message='foo'),
            Event(level='error', message='foo'),
            Event(level='error', message='foo', exc_info=True),
            Event(level='critical', message='foo'),
            Event(level='critical', message='foo'),
        )

    def test_includes_magic_vars(self):
        # built-in wins!
        self.logger.debug('foo', 'bar', 'baz', message='bob', args='er?')
        with ShouldRaise(TypeError):
            self.logger.log(42, 'foo', 'bar', 'baz',
                            message='bob', args='er?', level=50)
        self.check(
            Event(level='debug', message='foo', args=('bar', 'baz')),
        )

    def test_bind(self):
        logger = self.logger.bind(name='my.logger')
        logger.debug('foo')
        self.logger.debug('bar')
        self.check(
            Event(name='my.logger', level='debug', message='foo'),
            Event(level='debug', message='bar'),
        )

    def test_nested_bind(self):
        log = self.logger.bind(keep=1, replace=2)
        log.info(event='first bind')
        log = log.bind(replace=3, new=4)
        log.info(event='second bind')
        self.check(
            Event(level='info', event='first bind', keep=1, replace=2),
            Event(level='info', event='second bind', keep=1, replace=3, new=4),
        )

    def test_alter(self):
        self.logger.info('foo')
        self.logger.alter(name='my.logger')
        self.logger.info('bar')
        self.check(
            Event(level='info', message='foo'),
            Event(name='my.logger', level='info', message='bar'),
        )



from unittest import TestCase

from testfixtures import LogCapture, compare

from shoehorn import get_logger
from shoehorn.compat import PY2
from shoehorn.event import Event

logger = get_logger()


class TestStandardLibraryTarget(TestCase):

    def setUp(self):
        self.capture = LogCapture(
            attributes=('name', 'levelname', 'getMessage', 'shoehorn_event')
        )
        self.addCleanup(self.capture.uninstall)

    def test_minimal(self):
        logger.info(event='test')
        self.capture.check(
            ('root', 'INFO', 'None',
             Event(event='test', level='info'))
        )

    def test_named_logger(self):
        logger = get_logger('foo')
        logger.info(event='test')
        self.capture.check(
            ('foo', 'INFO', 'None',
             Event(event='test', logger='foo', level='info'))
        )

    def test_level(self):
        logger.warning(event='test')
        self.capture.check(
            ('root', 'WARNING', 'None',
             Event(event='test', level='warning'))
        )

    def test_sub_args(self):
        logger.info('foo %s', 'bar')
        self.capture.check(
            ('root', 'INFO', 'foo bar',
             Event(message='foo %s', args=('bar', ), level='info'))
        )

    def test_exc_info(self):
        bad = Exception('bad')
        try:
            raise bad
        except:
            logger.exception('foo')
        self.capture.check(
            ('root', 'ERROR', 'foo',
             Event(level='error', message='foo', exc_info=True))
        )
        compare(bad, actual=self.capture.records[-1].exc_info[1])

    def test_stack_info(self):
        if PY2:
            return
        logger.info('foo', stack_info=True)
        self.capture.check(
            ('root', 'INFO', 'foo',
             Event(message='foo', stack_info=True, level='info'))
        )
        compare('Stack (most recent call last):',
                actual=self.capture.records[-1].stack_info.split('\n')[0])

    def test_default_logger(self):
        from shoehorn import logger
        logger.info('er hi')
        self.capture.check(
            ('root', 'INFO', 'er hi',
             Event(message='er hi', level='info'))
        )



from unittest import TestCase

from testfixtures import compare

from shoehorn.event import Event


class TestEvent(TestCase):

    def test_repr(self):
        compare(repr(Event(y=1, x='2')), expected="Event(x='2', y=1)")

    def test_str(self):
        compare(str(Event(x=1, y=2)), expected="Event(x=1, y=2)")



from shoehorn.stdlib import StandardLibraryTarget
from .logger import Logger

logger = Logger(StandardLibraryTarget())


def get_logger(name=None):
    context = {}
    if name is not None:
        context['logger'] = name
    return logger.bind(**context)



# See license.txt for license details.
# Copyright (c) 2016 Chris Withers

import os

from setuptools import setup, find_packages

base_dir = os.path.dirname(__file__)

setup(
    name='shoehorn',
    version='0.1.5',
    author='Chris Withers',
    author_email='chris@withers.org',
    license='MIT',
    description="Shoehorn structured logs into or out of standard library logging",
    # long_description=open('docs/description.rst').read(),
    # url='https://github.com/cjw296/chide',
    classifiers=[
        # 'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    packages=find_packages(exclude='tests'),
    zip_safe=False,
    include_package_data=True,
    extras_require=dict(
        test=[
            'coveralls',
            'manuel',
            'nose',
            'nose-cov',
            'nose-fixes',
            'testfixtures',
            ],
        build=['sphinx', 'pkginfo', 'setuptools-git', 'twine', 'wheel']
    ),
)
